
package sub_arctic.input;

import sub_arctic.lib.interactor;

/** 
 * Input protocol for misc. dragging.  This protocol is designed for generic
 * dragging (i.e., anything not handled by one of the more specifically 
 * targeted drag protocols).  It basically just passes in the various events
 * (which as usual are in the coordinate system of the object they are passed
 * to).  This protocol particularly suitable for dragging components that are
 * inside the object itself (such as the thumb of a slider).
 *
 * @author Scott Hudson
 */
public interface simple_draggable extends focusable, interactor {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Dispatch the start event of a simple drag.  
   *
   * @param event  evt       the event "causing" the start of the drag 
   *                         (normally a mouse button press).
   * @param Object user_info the uninterpreted user info that was associated
   *                         with this object when it became the drag focus.
   * @return boolean indicating whether the input was consumed.
   */
  public boolean drag_start(event evt, Object user_info);

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Dispatch a movement event during a simple drag.  
   *
   * @param event  evt       the event "causing" the the drag (normally a 
   *                         mouse move).
   * @param Object user_info the uninterpreted user info that was associated
   *                         with this object when it became the drag focus.
   * @return boolean indicating whether the input was consumed.
   */
  public boolean drag_feedback(event evt, Object user_info);

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Dispatch event corresponding to the end of a simple drag.  
   *
   * @param event  evt       the event "causing" the end of the drag (normally 
   *                         a mouse button release).
   * @param Object user_info the uninterpreted user info that was associated
   *                         with this object when it became the drag focus.
   * @return boolean indicating whether the input was consumed.
   */
  public boolean drag_end(event evt, Object user_info);

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
