
package sub_arctic.lib;

import java.awt.Point;
import java.awt.Dimension;

/** 
 * Base class for objects that support (arbitrary) children.  This is basically
 * just base_interactor with child functionality added.  Normally all objects
 * supporting children are subclasses of this class.  However, the actual
 * routines needed to establish and support children have been promoted to
 * base_interactor.  As a result, if you have an existing non-parent class 
 * that you would like to add parent capability to after the fact that is 
 * also possible.  This class can serve as a guide for how to do that -- 
 * basically you just need to call setup_for_children() or 
 * setup_for_fixed_children() in the constructor.

 * @see sub_arctic.lib.base_interactor#setup_for_children
 * @see sub_arctic.lib.base_interactor#setup_for_fixed_children
 *
 * @author Scott Hudson
 */
public class base_parent_interactor extends base_interactor {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Full constructor 
   * @param int xv x position of the interactor in its parent's coordinate 
   *               system.
   * @param int yv y position of the interactor in its parent's coordinate 
   *               system.
   * @param int wv width of the interactor.
   * @param int hv height of the interactor.
   */
  public base_parent_interactor(int xv, int yv, int wv, int hv)
    {
      super(xv,yv,wv,hv);
      setup_for_children();
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Constructor assuming a (temporary) default size 
   * @param int xv x position of the interactor in its parent's coordinate 
   *               system.
   * @param int yv y position of the interactor in its parent's coordinate 
   *               system.
   */
  public base_parent_interactor(int xv, int yv)
    {
      super(xv,yv);
      setup_for_children();
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Constructor that assumes a position of 0,0 and a (temporary) default
   * size. 
   */
  public base_parent_interactor()
    {
      super();
      setup_for_children();
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

}

/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
