package sub_arctic.lib;
import sub_arctic.input.event;

/**
 * This class implements the vertical scrollbar for a listbox. It is
 * subclassed from v_scrollbar and its only modification is that it
 * understands that the buttons of the vertical scrollbar should cause
 * the system to move to the next object in the list. <P>
 *
 * This is also a reasonable demo of how to get custom behavior out
 * of the scrollbars' buttons.
 *
 * @author Ian Smith
 */

public class listbox_v_scrollbar extends v_scrollbar {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** The listbox we are attached to. */
  protected listbox_display _display;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Create a vertical listbox scrollbar.  We don't bother trying to
   * size or position this object since the display will do that 
   * anyway.
   *
   * @param listbox_display disp the object we are working on behalf of
   */
  public listbox_v_scrollbar(listbox_display disp) {
    super(disp);
    _display=disp;
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This function gets called when the up button gets pressed.
   * @param event evt the event responsible
   */
  protected void handle_up_button(event evt) {
    _display.move_up();
    /* do callback */
    dynamic_callback(evt);
    static_callback(evt);
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This function gets called when the down button gets pressed.
   * @param event evt the event responsible
   */
  protected void handle_down_button(event evt) {
    _display.move_down();
    dynamic_callback(evt);
    static_callback(evt);
  }

   //had:
   //* @exception general PROPAGATED

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
}

/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
