package sub_arctic.demo_apps;

import sub_arctic.lib.object_list_element;
import sub_arctic.input.event;
import sub_arctic.input.display_help;
import sub_arctic.input.pick_collector;
import sub_arctic.input.pointable;
import sub_arctic.lib.column;
import sub_arctic.lib.manager;
import sub_arctic.lib.label;

import java.awt.Point;

/**
 * This is a special version of the object_list_element which knows about
 * being in to do lists.
 */
public class todo_list_element extends object_list_element implements display_help {
  /**
   * This is the mail message we are displaying.
   */
  protected message _msg;
  /**
   * This is the applet we are in.
   */
  protected mail_test _applet;
  /**
   * This column gets used to display some labels 
   */
  protected column _label_parent=null;
  /**
   * Store an X location for the big message
   */
  protected int _msg_x;
  /**
   * Store a Y location for the big message
   */
  protected int _msg_y;
  /**
   * Create a todo_list_element from a message and our applet.
   */
  public todo_list_element(message obj, mail_test applet, 
			   int store_x, int store_y) {
    super(obj);
    _msg=obj;
    _applet=applet;
    _msg_x=store_x;
    _msg_y=store_y;
  }
  /**
   * Handle the pointable interface... on mouse enter we put a message
   * in the status window.
   */
  public boolean mouse_enter(event evt, Object user_info) {
    if (evt.id()!=event.MOUSE_MOVE) return false;
    _applet.showStatus("From: " + _msg.from());
    return true;
  }
  /**
   * Handle the pointable interface... on mouse exit we do nothing.
   */
  public void mouse_exit(event evt) {
  }
  /**
   * Handle the display help interface.
   */
  public void help_open() {
    label to, from, subject, date;
    Point pt;
    /* sanity check .. this shouldn't happen*/
    if (_label_parent!=null) {
      return;
    }
    /* where are we in our top level parent?*/
    pt=new Point(0,h()+1);
    pt=local_to_global(pt);
    /* build the column */
    _label_parent=new column(0,column.LEFT_JUSTIFIED);
    /* we want a little bit of a box*/
    _label_parent.set_boxed(true);
    /* build the labels */
    to=new label("To: " + _msg.to());
    from=new label("From: " + _msg.from());
    subject=new label("Subject: " + _msg.subject());
    date=new label("Date: " + _msg.date());
    /* add them */
    _label_parent.add_child(from);
    _label_parent.add_child(to);
    _label_parent.add_child(subject);
    _label_parent.add_child(date);
    /* set the coords so its set up right */
    _label_parent.set_x(pt.x);
    _label_parent.set_y(pt.y);
    get_top_level().add_child(_label_parent);
  }
  /**
   * Handle taking down the extra info
   */
  public void help_close(event evt) {
    /* sanity check */
    if (_label_parent==null) {
      return;
    }
    /* take it down */
    get_top_level().remove_child(_label_parent);
    _label_parent=null;
  }
  /**
   * Restore the big message to its place out in the playing field.
   */
  public void restore_message() {
    /* reconstruct the object */
    message_display display=new message_display(_msg,_applet);
    _applet.playing_field().add_child(display);
    /* reset it on the display */
    /* if they don't have a location use 20,20 */
    if (_msg_x==-1) {
      display.set_x(20);
      display.set_y(20);
    } else {
      /* they have a location */
      display.set_x(_msg_x);
      display.set_y(_msg_y);
    }
    /* resize it */
    header_listbox.size_message(display);
  }
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
