package sub_arctic.input;

/** 
 * Input protocol interface for receiving double clicks (two clicks in a small
 * area within a short period of time).  This protocol is dispatched by the 
 * click_agent.  See the click agent for a description of interactions between
 * this protocol and the pressable and clickable protocols.<p>
 *
 * @see sub_arctic.input.click_agent
 * @author Scott Hudson
 */
public interface double_clickable {
  /** 
   * Dispatch double click (two clicks in a small area within a short period of
   * time) to the object.  
   * @param event  evt       the final event in the double click sequence.
   * @param Object user_info uninterpreted user information that was originally
   *                         given to the system when this object was picked.
   * @return boolean indicating whether the object consumed the input.
   */
  public boolean double_click(event evt, Object user_info);
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
