
package sub_arctic.input;

import sub_arctic.lib.interactor;
import java.awt.Point;

/** 
 * Interface for objects that filter and transform points for purposes of 
 * limiting or transforming the action of a move-drag.  This interface
 * provides a simple API with one method which takes a point and returns
 * a transformed point.  
 * @see sub_arctic.input.move_draggable
 * @see sub_arctic.input.move_drag_focus_agent
 * @author Scott Hudson
 */
public interface move_drag_filter {

  /** 
   * Filter a single point.  This is done for move-dragging of an object.
   * In addition to the point being filtered (which is expressed in 
   * the parent coordinates of the object being dragged), this takes a 
   * reference to the object being dragged, and the current feature point 
   * of that object (expressed in the local coordinates of the object).  The 
   * feature point represents the position within the object that is being 
   * filtered or limited (this typically defaults to the top-left corner, 
   * for example). 
   * @param original_pt the point to be filtered (in parent's coords)
   * @param drag_obj    the object being dragged
   * @param feature_pt  the feature point within the object being dragged 
   *                    that is being filtered (in object's coords)
   * @return the location of the filtered point 
   */
  public Point filter_pt(
    Point      original_pt, 
    interactor drag_obj, 
    Point      feature_pt);
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
