
package sub_arctic.lib;

import sub_arctic.lib.interactor;

/** 
 * Interface defining the API for custom cycle handlers.  Objects of this 
 * type can be installed with the manager to get called when a cycle in the 
 * constraint system occurs.
 *
 * @see sub_arctic.lib.manager#handle_cycles_with
 * @author Scott Hudson
 */
public interface cycle_handler {
  /** 
   * Do something in response to a cycle.  This should return true if 
   * evaluation should proceed normally, and false the existing attribute 
   * value (perhaps assigned by this routine) should be left in place, but 
   * marked up-to-date.
   *
   * @param interactor in_obj    the object within which the cycle was first 
   *                             detected.
   * @param int        part_code the part of that object involved in the cycle.
   * @return boolean indicating if normal evaluation should proceed.
   */
  public boolean handle_cycle(interactor in_obj, int part_code);
}

/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
