package sub_arctic.lib;

import sub_arctic.input.callback_object;
import sub_arctic.constraints.std_constraint_consts;

import java.awt.Frame;
import java.awt.Event;
import java.awt.Dimension;
import java.applet.Applet;
import java.awt.Component;

/**
 * This class right now is basically a placeholder for the top-level object
 * of a stand-alone application. It is intended that later on this class will 
 * provide you with many mechanisms for doing fun things on a per-application 
 * basis. <P>
 * 
 * For now, please just subclass this object when doing your application
 * development. Stay tuned...<P>
 * @author Ian Smith
 */
public class interactor_app implements std_constraint_consts {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Arguments from the command line.
   */
  protected String[] _argv;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Access the command line arguments.
   * @return String[] the arguments from the user on the command line
   */
  public String[] command_line_arguments() { return _argv;}

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Construct an interactor app given an array of strings, presumably
   * those passed in from the command line.     
   * @param String[] argv the array of arguments passed from the operating 
   *                      system.
   */
  public interactor_app(String[] argv) {
    /* right now there's not much to do*/
    _argv=argv;
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Parse the command line arguments supplied. In a future version,
   * this may contain automatic (table driven?) mechanisms for
   * parsing arguments and extracting values.  Right now this does nothing but
   * return true.
   *
   * @param String[] argv an array of arguments to parse
   * @return boolean return true if the arguments supplied result in a safe 
   *                 state for the application to proceed, return false if not.
   */
  public boolean parse_arguments(String[] argv) { return true;}

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This is the function you should call to begin your application.
   * It performs various toolkit and application level initialization
   * and then performs the following sequence of calls:
   * <DL>
   * <DT> parse_arguments() (
   * <DD> Parse the arguments supplied. If this call returns false, the 
   *      following stages are skipped)
   * <DT> app_initialize() 
   * <DD> Initialize your data structures here, assuming command line 
   *      arguments parsed ok
   * <DT> frame_initialize()
   * <DD> Perform creation of your applications frame(s) here
   * 
   * @return boolean returns true if the application's command line arguments 
   *                 are reasonable, false otherwise
   */
  public boolean initialize() {
    boolean ok=parse_arguments(_argv);
    if (ok) {
      app_initialize();
      frame_initialize();
    }
    return ok;
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Initialize your applications data structures here. You can assume
   * that the command line arguments have already been processed
   * and were parsed successfully.
   */
  public void app_initialize() {}

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Initialize your applications frames here. You can assume
   * that the command line arguments have already been processed
   * and were parsed successfully.
   */
  public void frame_initialize() {}

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
