
package sub_arctic.lib;

import java.awt.Rectangle;

/** 
 * An interface_pred predicate class to serve as a "continue predicate" (see
 * base_interactor.traverse_and_collect()) for a semantic redraw traversal 
 * (done by a semantic lens object).  We indicate that traversal should 
 * continue unless the parent object is trivially rejected with respect to 
 * the drawable contained in the sem_draw_context parameter.
 *
 * @see sub_arctic.lib.base_interactor#traverse_and_collect
 * @see sub_arctic.lib.semantic_lens
 * @see sub_arctic.lib.sem_draw_context
 * @author Scott Hudson
 */
public class sem_draw_continue implements interactor_pred {
  /** Perform the predicate test.
   *  
   *  @param obj        the interactor the predicate is testing.
   *  @param parameters a sem_draw_context object containing the drawable whose
   *                    clipping area we are against the given object.
   *  @return predicate result.
   */
  public boolean test(interactor obj, Object parameters) 
    {
      Rectangle bound;

      /* type check the parameters */
      if (!(parameters instanceof sem_draw_context))
	throw new sub_arctic_error("Non-sem_draw_context object passed "+
				   "as parameter");

      /* do the test */
      bound = new Rectangle(0,0,obj.w(), obj.h());
      return ((sem_draw_context)parameters).surface.getClipRect().
						    intersects(bound);
    }

   //had:
   //*  @exception sub_arctic.exception.bad_value thrown if the parameters 
   //*     argument is not a sem_draw_context.
};
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
